/** @file
 * @brief Virtual serial interface via USB.
 * 
 * Virtual serial interface uses CDC via USB for serial communication.
 */
/*
 * $LastChangedRevision: 853 $
 * $LastChangedDate:: 2015-11-26 15:56:03 +0100#$
 *
 * Copyright (c) 2015 Linz Center of Mechatronics GmbH
 * All rights reserved.
 */
#ifndef __VIRTUALSERIALPIC32MZ_H__
#define __VIRTUALSERIALPIC32MZ_H__

#include "system_config.h"
#include "system_definitions.h"
#include "CommonFcts.h"
#include "Target.h"

typedef enum
{
    /* Application's state machine's initial state. */
    COM_STATE_INIT=0,

    /* Application waits for device configuration*/
    COM_STATE_WAIT_FOR_CONFIGURATION,

    /* Wait for a character receive */
    COM_STATE_SCHEDULE_READ,

    /* A character is received from host */
    COM_STATE_WAIT_FOR_READ_COMPLETE,

    /* Wait for the TX to get completed */
    COM_STATE_SCHEDULE_WRITE,

    /* Wait for the write to complete */
    COM_STATE_WAIT_FOR_WRITE_COMPLETE,

    /* Application Error state*/
    COM_STATE_ERROR

} VIRTUAL_COM_STATE;

typedef struct {
	void (*send)(tInterface*, uint8);               /** Function link to send character */
	uint8 (*receive)(tInterface*);                  /** Function link to receive character */
	uint8 (*isReceiveDataAvailable)(tInterface*);   /** Function link to check if receive data is available */
	uint8 (*isSendReady)(tInterface*);              /** Function link to check if data can be sent */
	uint8 (*getTxFifoFree)(tInterface*);            /** Function link to check if transmit FIFO is ready to be filled */
	void (*flush)(tInterface*);                     /** Function link to flush data */
    
//    uint8* rxBuffer;
//    uint16 rxBufferSize;
//    uint8* txBuffer;
//    uint16 txBufferSize;
    
    /* TODO USB specific parameters */
        /* Device layer handle returned by device layer open function */
    USB_DEVICE_HANDLE deviceHandle;

    /* Application's current state*/
    VIRTUAL_COM_STATE state;

    /* Set Line Coding Data */
    USB_CDC_LINE_CODING setLineCodingData;

    /* Device configured state */
    bool isConfigured;

    /* Get Line Coding Data */
    USB_CDC_LINE_CODING getLineCodingData;

    /* Control Line State */
    USB_CDC_CONTROL_LINE_STATE controlLineStateData;

    /* Read transfer handle */
    USB_DEVICE_CDC_TRANSFER_HANDLE readTransferHandle;

    /* Write transfer handle */
    USB_DEVICE_CDC_TRANSFER_HANDLE writeTransferHandle;

    /* True if a character was read */
    bool isReadComplete;

    /* True if a character was written*/
    bool isWriteComplete;

    /* Flag determines SOF event occurrence */
    bool sofEventHasOccurred;

    /* Break data */
    uint16_t breakData;

    /* Switch debounce timer */
    unsigned int switchDebounceTimer;

    unsigned int debounceCount;
} tVirtualSerial;

/* public prototypes */
void initVirtualSerial(void);

#endif
